import org.http4k.core.Body
import org.http4k.core.HttpHandler
import org.http4k.core.Method
import org.http4k.core.Response
import org.http4k.core.Status.Companion.FOUND
import org.http4k.core.Status.Companion.OK
import org.http4k.core.body.form
import org.http4k.core.getFirst
import org.http4k.core.toParametersMap
import org.http4k.core.with
import org.http4k.lens.BiDiBodyLens
import org.http4k.lens.FormField
import org.http4k.lens.Validator
import org.http4k.lens.localDate
import org.http4k.lens.localTime
import org.http4k.lens.string
import org.http4k.lens.webForm
import org.http4k.routing.bind
import org.http4k.routing.routes
import org.http4k.template.ViewModel
import ru.ac.uniyar.domain.Schedule
import ru.ac.uniyar.domain.WorkingHours
import ru.ac.uniyar.models.ShowWorkingHoursFormViewModel
import java.time.LocalDate
import java.time.LocalTime

fun scheduleCreationRoute(schedule: Schedule, htmlView: BiDiBodyLens<ViewModel>) = routes(
    "/" bind Method.GET to showWorkingHoursForm(htmlView),
    "/" bind Method.POST to addWorkingHoursWithLens(schedule, htmlView)
)

fun showWorkingHoursForm(htmlView: BiDiBodyLens<ViewModel>): HttpHandler = {
    Response(OK).with(htmlView of ShowWorkingHoursFormViewModel())
}

val dateFormLens = FormField.localDate().required("date")
val startFormLens = FormField.localTime().required("start")
val endFormLens = FormField.localTime().required("end")
val descriptionFormLens = FormField.string().required("description")
val workingHoursFormLens = Body.webForm(
    Validator.Feedback,
    dateFormLens,
    startFormLens,
    endFormLens,
    descriptionFormLens,
).toLens()

fun addWorkingHoursWithLens(schedule: Schedule, htmlView: BiDiBodyLens<ViewModel>): HttpHandler = { request ->
    val webForm = workingHoursFormLens(request)
    if(webForm.errors.isEmpty()) {
        schedule.add(WorkingHours(
            dateFormLens(webForm),
            startFormLens(webForm),
            endFormLens(webForm),
            descriptionFormLens(webForm),
        ))
        Response(FOUND).header("Location", "/schedule")
    } else {
        Response(OK).with(htmlView of ShowWorkingHoursFormViewModel(webForm))
    }
}

fun addWorkingHours(schedule: Schedule): HttpHandler = { request ->
    val form = request.form().toParametersMap()
    val dateString = form.getFirst("date").orEmpty()
    val startString = form.getFirst("start").orEmpty()
    val endString = form.getFirst("end").orEmpty()
    val descriptionString = form.getFirst("description").orEmpty()
    val workingHours = WorkingHours(
        LocalDate.parse(dateString),
        LocalTime.parse(startString),
        LocalTime.parse(endString),
        descriptionString
    )
    schedule.add(workingHours)
    Response(FOUND).header("Location", "/schedule")
}
